
/**
* Kryptographie-Klasse, die eine leicht zu bedienende Schnittstelle zu einem 
* konkreten Verschlsselungsverfahren (hier: RSA) darstellt.
* Der Einfachheit halber arbeiten alle Methoden mit Strings. Verschlsselte Nachrichten
* werden als Zahlenkolonnen (hexadezimal!) dargestellt (d.h. sie enthalten nur Zeichen
* '0'-'9' und 'A'-'F' und insbesondere keinen Zeilenumbruch). Somit bleiben 
* verschlsselte Nachrichten 'lesbar'.
*
* @version 1.0
* @author Thomas Wolf
*/
public class Cryptography {

	/** Kryptographie-Algorithmus. */
	private RSA cryptEngine=null;

	/**
	* Default-Konstruktor.
	*/
	public Cryptography() {
		cryptEngine=new RSA();
	}

	/**
	* Erzeugt einen neuen Schlssel (privat und ffentlich). Die Ausfhrung 
	* dieser Methode bentigt je nach Rechner ein paar Sekunden/Minuten Zeit.
	*/
	public void generateKey() {
		cryptEngine.generateKey();
	}
	/**
	* Liefert den ffentlichen Schlssel zurck.
	* @return ffentlicher Schlssel
	*/
	public String getKey() {
		return bytes2String(cryptEngine.getPublicKey());
	}
	/**
	* Setzt den Schlssel auf den Wert von key.
	* @param key neuer Schlssel
	*/
	public void setKey(String key) {
		cryptEngine.setKey(string2Bytes(key));
	}
	/**
	* Verschlsselt den Text s und gibt den chiffrierten Text zurck.
	* @param s Nachricht
	* @return verschlsselte Nachricht
	*/
	public String encrypt(String s) {
		return bytes2String(cryptEngine.encrypt(s.getBytes()));
	}
	/**
	* Entschlsselt die Nachricht s.
	* @param s verschlsselte Nachricht
	* @return entschlsselter Text
	*/
	public String decrypt(String s) {
		return new String(cryptEngine.decrypt(string2Bytes(s)));
	}
	/**
	* Wandelt ein Array von Bytes in einen String um, der nur die
	* Zeichen '0'-'9' und 'A'-'F' enthlt.
	* @param array umzuwandelndes Byte-Array
	* @return String (hat doppelte Lnge, wie das Array)
	*/
	private String bytes2String(byte[] array) {
		if ((array==null) || (array.length<=0)) return new String();
		char[] ch=new char[2*array.length];
		char[] ct;
		for(int i=0;i<array.length;i++) {
			ct=byte2HexCharArray(array[i]);
			ch[2*i]=ct[0]; ch[2*i+1]=ct[1];
		}
		return new String(ch);
	}
	/**
	* Wandelt einen codierten String, der nur die Zeichen '0'-'9' und 'A'-'F'
	* enthlt in ein Byte-Array (halber Lnge) um.
	* @param s String
	* @return Byte-Array
	*/
	private byte[] string2Bytes(String s) {
		char[] ch=s.toCharArray();
		byte[] bytes=new byte[ch.length/2];
		for(int i=0;i<bytes.length;i++)
			bytes[i]=charArray2Byte(ch,2*i);
		return bytes;
	}
	/**
	* Wandelt ein Byte in seine hexadezimale Darstellung als Char-Array der
	* Lnge zwei um.
	* @param b Byte
	* @return Char-Array der Lnge zwei mit der hexadezimalen Darstellung von b
	*/
	private char[] byte2HexCharArray(byte b) {
		byte b1=(byte)(((b & 0xf0)>>4)+0x30);
		byte b2=(byte)((b & 0x0f)+0x30);
		if (b1>0x39) b1+=7;
		if (b2>0x39) b2+=7;
		char[] c=new char[2]; c[0]=(char)b1; c[1]=(char)b2;
		return c;
	}
	/**
	* Liest zwei Zeichen vom Array c ab Index i ein und fat es als
	* hexadezimaldarstellung eines Bytes auf, das schlielich zurckgegeben wird.
	* @param c Char-Array
	* @param i Start-Index
	* @return byte, dessen hexadezimale Darstellung ab i auf c steht
	*/
	private byte charArray2Byte(char[] c,int i) {
		byte b1=(byte)(c[i]-0x30),b2=(byte)(c[i+1]-0x30);
		if (b1>16) b1-=7;
		if (b2>16) b2-=7;
		return (byte)((b1<<4)+b2);
	}

}