import java.math.BigInteger;
import java.util.*;

/**
* RSA-Verschlsselung.
*
* @author Thomas Wolf
* @version 1.0
*/
public class RSA {

	/** Produkt zweier (groer) Primzahlen. */
	private BigInteger n=null;
	/** ffentlicher Teil des Schlssels */
	private BigInteger e=null;
	/** privater Teil des Schlssels. */
	private BigInteger d=null;

	/** Die Zahl 1 als BigInteger-Konstante. */
	private final static BigInteger ONE=new BigInteger("1");

	/** maximale Lnge eines codierten Blockes */
	public final static int BLOCK_LENGTH=129;
	/** maximale Lnge eines zu lesenden Blockes */
	public final static int BLOCK_SIZE=BLOCK_LENGTH-4;

	/** Zufallszahlengenerator fr das Erzeugen von Schlsseln und
	*   das Auffllen von Byte-Arrays. */
	private Random rnd=null;

	/**
	* Default-Konstruktor.
	*/
	public RSA() {
		n=e=d=null;
		// Zufallszahlengenerator erzeugen
		rnd=new Random();
	}

	/**
	* Erzeugt einen (privaten und ffentlichen) Schlssel.
	*/
	public void generateKey() {
		// erzeugt mit Wahrscheinlichkeit 1-2^20 eine Primzahl
		BigInteger p=new BigInteger(513,20,rnd);
		BigInteger q;
		
		// p und q mssen ungleich sein!
		do {
		 q=new BigInteger(513,20,rnd);
		} while (p.equals(q));

		// n, the modulus, is p*q.
		n=p.multiply(q);

		// To work out d, we need to know (p-1)*(q-1)
		p=p.subtract(ONE);
		q=q.subtract(ONE);
		BigInteger pq = p.multiply(q);

		// Without loss of security, we can just set e to 2^16+1.
		// In fact, this is the recommendation in X.509!
		e=new BigInteger("65537");
		//e=new BigInteger(100, new Random());
  //do { 	e=e.add(ONE); } while(!(e.gcd(pq)).equals(ONE));  

		// d is the inverse of e modulo pq.
		d=e.modInverse(pq);
	}

	/**
	* Setzt den Schlssel.
	* @param key byte-Array, das den Schlssel reprsentiert.
 */
	public void setKey(byte[] key) {
		if ((key==null) || ((key.length!=2*BLOCK_LENGTH) && (key.length!=3*BLOCK_LENGTH)))
			throw new EncryptionException(EncryptionException.CR_WRONG_KEY);
		n=readCodeBlock(key,0);
		e=readCodeBlock(key,BLOCK_LENGTH);
		if (key.length==3*BLOCK_LENGTH) {
			d=readCodeBlock(key,2*BLOCK_LENGTH);
		} else d=null;
	}


	/**
	* Liefert den ffentlichen Teil des Schlssels. Wird ein solcher
	* Schlssel mit <strong>setKey()</strong> benutzt, so kann nur noch
	* encrypt fehlerfrei durchgefhrt werden.
	* @return ffentlicher Schlssel
 */
	public byte[] getPublicKey() {
		byte[] key=new byte[2*BLOCK_LENGTH];
		writeCodeBlock(n,key,0);
		writeCodeBlock(e,key,BLOCK_LENGTH);
		return key;
	}
	/**
	* Liefert den gesamten Schlssel.
	* @return Schlssel
 */
	public byte[] getKey() {
		byte[] key=new byte[3*BLOCK_LENGTH];
		writeCodeBlock(n,key,0);
		writeCodeBlock(e,key,BLOCK_LENGTH);
		writeCodeBlock(d,key,2*BLOCK_LENGTH);
		return key;
	}


	/**
	* Grundlegende RSA-Operation (m^exp mod n).
	* @param m Nachricht als BigInteger
	* @param exp zu benutzender Teilschlssel (je nachdem ob ver- oder entschlsseln)
	* @return m^exp mod n
	*/
	private BigInteger rsa(BigInteger m,BigInteger exp) {
		return m.modPow(exp,n);
	}

	/**
	* Liest einen Block der Gre BLOCK_SIZE (oder weniger, falls message zu kurz) vom
	* array message ab dem Index idx und generiert eine positive Zahl kleiner als n
	* daraus, indem Teile durch Zufallszahlen ausgefllt werden.
	* @param message array mit den Daten
	* @param idx Index, ab dem gelesen wird
	* @return generierte Zahl
	*/
	private BigInteger plain2BigInt(byte[] message,int idx)	{
		// Nachrichtenlnge
		int len=Math.min(BLOCK_SIZE,message.length-idx);
		// aufgefllte Nachricht
		byte[] pad=new byte[BLOCK_LENGTH-1];
		// rechtsbndig auffllen
		System.arraycopy(message,idx,pad,BLOCK_LENGTH-1-len,len);
		// Beim Konvertieren von m in einen BigInteger soll die Zahl positiv(!)
		// und in der Grenordnung von n sein. Also setzen wir das
		// erste Byte auf einen Zufallswert zwischen 0x40 und 0x7f.
		// (das Array ist in bigEndian-Order!)
		pad[0]=(byte)(0x40+(rnd.nextInt() & 0x3f));
		// Die Bytes 1 und 2 enthalten die Lnge der Nachricht als short.
		pad[1]=(byte)((len >>> 8) & 0xff);
		pad[2]=(byte)(len & 0xff);

		// Den Rest mit Zufallszahlen auffllen
		byte[] padJunk=new byte[BLOCK_LENGTH-4-len];
		rnd.nextBytes(padJunk);
		System.arraycopy(padJunk,0,pad,3,padJunk.length);
		return new BigInteger(pad);
	}
	
	/**
	* Erzeugt aus einer Zahl, die mit <strong>plain2BigInt()</strong> erzeugt
	* wurde, wieder ein Byte-Array mit der Original-Nachricht. Das Array ist
	* maximal BLOCK_SIZE lang.
	* @param m Zahl
	* @return Byte-Array mit original Nachrichtenblock
 */
	private byte[] bigInt2Plain(BigInteger m) {
		byte[] message=m.toByteArray();
		// Bei erfolgreicher Decodierung enthalten die Bytes 1 und 2
		// die Lnge der Nachricht als short, ansonsten ging etwas 
		// schief und wir lsen eine Exception aus.
		int len=(((short)message[1]) << 8)+(short)message[2];
		if (len>(BLOCK_SIZE))
			throw new EncryptionException(EncryptionException.ES_WRONG_LENGTH);
		byte[] dest=new byte[len];
		System.arraycopy(message,BLOCK_LENGTH-1-len,dest,0,len);
		return dest;
	}
	/**
	* Schreibt eine codierte Zahl als Block der Lnge BLOCK_LENGTH	auf das 
	* Array dest ab dem Index idx.
	* @param z Zahl
	* @param dest Byte-Array, auf dem geschrieben wird
	* @param idx Index, ab dem geschrieben wird
	*/
	private void writeCodeBlock(BigInteger z,byte[] dest,int idx) {
		byte[] b=z.toByteArray();
		if (b.length>BLOCK_LENGTH)
			throw new EncryptionException(EncryptionException.CO_TOO_BIG);
		// Mit Nullen auffllen
		for(int i=0;i<BLOCK_LENGTH-b.length;i++) dest[idx+i]=(byte)0;
		System.arraycopy(b,0,dest,idx+BLOCK_LENGTH-b.length,b.length);
	}
	/**
	* Liest einen Block der Lnge BLOCK_LENGTH ab Index idx vom Byte-Array
	* source in eine Zahl.
	* @param source Array, von dem gelesen wird
	* @param idx Index, ab dem gelesen wird
	* @return gelesene Zahl
	*/
	private BigInteger readCodeBlock(byte[] source,int idx) {
		byte[] b=new byte[BLOCK_LENGTH];
		System.arraycopy(source,idx,b,0,BLOCK_LENGTH);
		return new BigInteger(b);
	}
	/**
	* Verschlsselt die Nachricht plain mit dem RSA-Verfahren und gibt die chiffrierte
	* Nachricht zurck.
	* @param plain Nachricht als Byte-Array
	* @return verschlsselte Nachricht
	*/
	public byte[] encrypt(byte[] plain) {
		if ((n==null) || (e==null))
			throw new EncryptionException(EncryptionException.CR_NO_KEY);
		// Anzahl ganzer Blcke
		int numblocks=plain.length/BLOCK_SIZE;
		byte[] cipher=new byte[((plain.length%BLOCK_SIZE==0)?(numblocks*BLOCK_LENGTH)
			                                                   :((numblocks+1)*BLOCK_LENGTH))];
		BigInteger m;
		int j=0;
		for(int i=0;i<plain.length;i+=BLOCK_SIZE) {
			// BigInteger mit geforderten Eigenschaften aus dem Text generieren...
			m=plain2BigInt(plain,i);
			// codieren...
			m=rsa(m,e);
			// BigInteger schreiben...
			writeCodeBlock(m,cipher,j); j+=BLOCK_LENGTH;
		}
		return cipher;
	}
	/**
	* Entschlsselt die Nachricht cipher mit dem RSA-Verfahren und gibt die 
	* dechiffrierten Nachricht zurck. Aufgrund der Schlsselwahl in 
	* <strong>generateKey()</strong> dauert Entschlsseln wesentlich lnger
	* als Verschlsseln!
	* @param cipher chiffrierte Nachricht als Byte-Array
	* @return dechiffrierte Nachricht
	*/
	public byte[] decrypt(byte[] cipher) {
		if ((n==null) || (e==null))
			throw new EncryptionException(EncryptionException.CR_NO_KEY);
		if (d==null) 
			throw new EncryptionException(EncryptionException.CR_NO_PRIVATE);
		if ((cipher==null) || (cipher.length<=0)) return new byte[0];
		// Die Lnge des Textes berechnen...
		// ... letzten Block dekodieren (alle anderen Blcke haben (dekodierte) Lnge BLOCK_SIZE)
		BigInteger m=rsa(readCodeBlock(cipher,cipher.length-BLOCK_LENGTH),d);
		int j=((cipher.length/BLOCK_LENGTH)-1)*BLOCK_SIZE+bigInt2Plain(m).length;
		byte[] plain=new byte[j],b;
		j=0;
		for(int i=0;i<cipher.length;i+=BLOCK_LENGTH) {
			// Code-Block lesen und dekodieren
			m=rsa(readCodeBlock(cipher,i),d);
			// wieder in Plain-Text umwandeln...
			b=bigInt2Plain(m);
			// und kopieren
			System.arraycopy(b,0,plain,j,b.length); j+=b.length;
		}
		return plain;
	}

}
